Package: MAPFX
Title: MAssively Parallel Flow cytometry Xplorer (MAPFX): A Toolbox for
        Analysing Data from the Massively-Parallel Cytometry
        Experiments
Version: 1.7.0
Authors@R: 
	c(person("Hsiao-Chi", "Liao", email = "chelsea.acad@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-9586-1246")),
	  person("Agus", "Salim", role = c("ctb")), person("infinityFlow", role = c("ctb")))
Description: MAPFX is an end-to-end toolbox that pre-processes the raw data from MPC experiments (e.g., BioLegend's LEGENDScreen and BD Lyoplates assays), and further imputes the ‘missing’ infinity markers in the wells without those measurements. The pipeline starts by performing background correction on raw intensities to remove the noise from electronic baseline restoration and fluorescence compensation by adapting a normal-exponential convolution model. Unwanted technical variation, from sources such as well effects, is then removed using a log-normal model with plate, column, and row factors, after which infinity markers are imputed using the informative backbone markers as predictors. The completed dataset can then be used for clustering and other statistical analyses. Additionally, MAPFX can be used to normalise data from FFC assays as well.
Depends: R (>= 4.4.0)
License: GPL-2
Encoding: UTF-8
RoxygenNote: 7.3.1
Roxygen: list(markdown = TRUE)
biocViews: Software, FlowCytometry, CellBasedAssays, SingleCell,
        Proteomics, Clustering
Imports: flowCore, Biobase, stringr, uwot, iCellR, igraph, ggplot2,
        RColorBrewer, Rfast, ComplexHeatmap, circlize, glmnetUtils,
        e1071, xgboost, parallel, pbapply, reshape2, gtools, utils,
        stats, cowplot, methods, grDevices, graphics
Suggests: BiocStyle, knitr, rmarkdown, testthat
VignetteBuilder: knitr
NeedsCompilation: no
URL: https://github.com/HsiaoChiLiao/MAPFX
BugReports: https://github.com/HsiaoChiLiao/MAPFX/issues
git_url: https://git.bioconductor.org/packages/MAPFX
git_branch: devel
git_last_commit: e9f080c
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
Packaged: 2025-10-31 22:41:50 UTC; biocbuild
Author: Hsiao-Chi Liao [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-9586-1246>),
  Agus Salim [ctb],
  infinityFlow [ctb]
Maintainer: Hsiao-Chi Liao <chelsea.acad@gmail.com>
