# Generated by roxygen2: do not edit by hand

export(MapfxFFC)
export(MapfxMPC)
import(ggplot2)
importFrom(Biobase,exprs)
importFrom(Biobase,pData)
importFrom(ComplexHeatmap,Heatmap)
importFrom(ComplexHeatmap,draw)
importFrom(ComplexHeatmap,rowAnnotation)
importFrom(RColorBrewer,brewer.pal)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(Rfast,colsums)
importFrom(Rfast,lmfit)
importFrom(circlize,colorRamp2)
importFrom(cowplot,plot_grid)
importFrom(e1071,svm)
importFrom(flowCore,keyword)
importFrom(flowCore,logicleTransform)
importFrom(flowCore,parameters)
importFrom(flowCore,read.flowSet)
importFrom(glmnetUtils,cv.glmnet)
importFrom(grDevices,dev.off)
importFrom(grDevices,jpeg)
importFrom(graphics,abline)
importFrom(gtools,combinations)
importFrom(iCellR,Rphenograph)
importFrom(igraph,membership)
importFrom(methods,is)
importFrom(parallel,clusterEvalQ)
importFrom(parallel,clusterExport)
importFrom(parallel,makeCluster)
importFrom(parallel,stopCluster)
importFrom(pbapply,pblapply)
importFrom(reshape2,melt)
importFrom(stats,"contrasts<-")
importFrom(stats,as.formula)
importFrom(stats,contr.sum)
importFrom(stats,dexp)
importFrom(stats,dnorm)
importFrom(stats,formula)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,optim)
importFrom(stats,pexp)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stringr,str_extract)
importFrom(utils,data)
importFrom(utils,getS3method)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,write.csv)
importFrom(uwot,umap)
importFrom(xgboost,"xgb.parameters<-")
importFrom(xgboost,xgb.Booster.complete)
importFrom(xgboost,xgboost)
